#!BPY

"""
Name: 'MeshAscii (*.mesh.ascii)'
Blender: 248
Group: 'Import'
"""

__author__ = "Dusan Pavlicek"
__email__ = "pavlicd@volny.cz"

from Blender import *
from Blender.Mathutils import *
import bpy

warningCounter = 0
rootDir = ''


def Main():
	Window.FileSelector(Import, 'Import', '*.mesh.ascii')
	return


def Import(filename):
	global rootDir
	rootDir = sys.dirname(filename)
	Window.WaitCursor(1)
	file = open(filename, 'r')
	boneCount = SkipBones(file)
	meshCount = int(ReadLineIgnoreComments(file))
	for i in range(0, meshCount):
		ImportMesh(file, boneCount > 0)
	file.close()
	Window.WaitCursor(0)
	Redraw()

	global warningCounter
	if warningCounter > 0:
		if warningCounter == 1:
			Draw.PupMenu('There has been 1 warning.')
		else:
			Draw.PupMenu('There have been %d warnings.' % warningCounter)
	return


def SkipBones(file):
	boneCount = int(ReadLineIgnoreComments(file))
	for i in range(0, boneCount):
		boneName = ReadLine(file)
		parentID = int(ReadLineIgnoreComments(file))
		tuple = ReadTuple(file) # relPosition
	return boneCount


def ImportMesh(file, hasBones):
	global warningCounter
	global rootDir
	meshName = ReadLine(file)
	mesh = bpy.data.meshes.new(meshName)
	mesh.vertexColors = 1

	uvLayerCount = int(ReadLineIgnoreComments(file))
	for i in range(0, uvLayerCount):
		mesh.addUVLayer('UVTex' + str(i + 1))

	material = Material.New(meshName)
	textureCount = int(ReadLineIgnoreComments(file))
	faceImage = None
	for textureID in range(0, textureCount):
		textureFilename = ReadLine(file)
		textureFilename = sys.join(rootDir, sys.basename(textureFilename))
		uvLayerID = int(ReadLineIgnoreComments(file))
		
		textureName = meshName + '_' + str(textureID + 1)
		texture = Texture.New(textureName)
		texture.type = Texture.Types.IMAGE
		image = None
		try:
			image = Image.Load(textureFilename)
		except:
			print 'Warning: Cannot load image \'' + textureFilename + '\''
			warningCounter += 1
		texture.image = image
		if textureID == 0:
			faceImage = image
		material.setTexture(textureID, texture)
		mtex = material.getTextures()[textureID]
		mtex.texco = Texture.TexCo.UV
		mtex.uvlayer = 'UVTex' + str(uvLayerID + 1)
	mesh.materials += [material]

	vertexCount = int(ReadLineIgnoreComments(file))
	coords = [[0, 0, 0]]
	normals = [[0, 0, 0]]
	colors = [[0, 0, 0, 0]]
	uvs = [[0, 0]]
	for vertexID in range(0, vertexCount):
		tuple = ReadTuple(file)
		x = float(tuple[0])
		y = float(tuple[1])
		z = float(tuple[2])
		coords.append([x, -z, y])
		
		tuple = ReadTuple(file)
		nx = float(tuple[0])
		ny = float(tuple[1])
		nz = float(tuple[2])
		normals.append([nx, -nz, ny])
		
		tuple = ReadTuple(file)
		r = int(tuple[0])
		g = int(tuple[1])
		b = int(tuple[2])
		a = int(tuple[3])
		colors.append([r, g, b, a])
		
		uvList = []
		for i in range(0, uvLayerCount):
			tuple = ReadTuple(file)
			u = float(tuple[0])
			v = float(tuple[1])
			uvList.append([u, 1 - v])
		uvs.append(uvList)

		if hasBones:
			tuple = ReadTuple(file) # boneIndices
			tuple = ReadTuple(file) # boneWeights

	mesh.verts.extend(coords)
	
	for vertexID in range(0, vertexCount + 1):
		mesh.verts[vertexID].no = Vector(normals[vertexID])
		
	faceCount = int(ReadLineIgnoreComments(file))
	indices = []
	for i in range(0, faceCount):
		tuple = ReadTuple(file)
		index1 = int(tuple[0])
		index2 = int(tuple[1])
		index3 = int(tuple[2])
		indices.append([index1 + 1, index3 + 1, index2 + 1])
	
	mesh.faces.extend(indices, ignoreDups=True)
	
	for faceID in range(0, faceCount):
		face = mesh.faces[faceID]
		face.image = faceImage
		index1 = indices[faceID][0]
		index2 = indices[faceID][1]
		index3 = indices[faceID][2]
		color1 = colors[index1]
		color2 = colors[index2]
		color3 = colors[index3]
		face.col[0].r = color1[0]
		face.col[0].g = color1[1]
		face.col[0].b = color1[2]
		face.col[0].a = color1[3]
		face.col[1].r = color2[0]
		face.col[1].g = color2[1]
		face.col[1].b = color2[2]
		face.col[1].a = color2[3]
		face.col[2].r = color3[0]
		face.col[2].g = color3[1]
		face.col[2].b = color3[2]
		face.col[2].a = color3[3]
		face.smooth = True

	for uvLayerID in range(0, uvLayerCount):
		mesh.activeUVLayer = 'UVTex' + str(uvLayerID + 1)
		for faceID in range(0, faceCount):
			face = mesh.faces[faceID]
			index1 = indices[faceID][0]
			index2 = indices[faceID][1]
			index3 = indices[faceID][2]
			uv1 = Vector(uvs[index1][uvLayerID])
			uv2 = Vector(uvs[index2][uvLayerID])
			uv3 = Vector(uvs[index3][uvLayerID])
			face.uv = [uv1, uv2, uv3]
	if uvLayerCount > 0:
		mesh.activeUVLayer = 'UVTex1'

	mesh.verts.delete(0)

	scene = bpy.data.scenes.active
	scene.objects.new(mesh, meshName)
	return


def ReadLine(file):
	line = file.readline()
	if line.endswith('\n'):
		line = line[:-1]
	if line.endswith('\r'):
		line = line[:-1]
	return line


def ReadLineIgnoreComments(file):
	line = ReadLine(file)
	pos = line.find('#')
	if pos >= 0:
		line = line[0:pos]
	return line


def ReadTuple(file):
	line = ReadLineIgnoreComments(file)
	return line.split()


Main()
